#ifndef _NF_SET_NETLINK_H
#define _NF_SET_NETLINK_H

/* nfset netlink message types */
enum nfset_msg_type {
	NFSET_MSG_CREATE,	/* Create a new (empty) set */
	NFSET_MSG_DESTROY,	/* Remove a (empty) set */
	NFSET_MSG_FLUSH,	/* Remove all elements from a set */
	NFSET_MSG_RENAME,	/* Rename a set */
	NFSET_MSG_SWAP,		/* Swap two sets */

	NFSET_MSG_HEADER,	/* Get set header (for ADT) */
	NFSET_MSG_LIST,		/* List sets */

	NFSET_MSG_ADD,		/* Add element(s) to a set */
	NFSET_MSG_DEL,		/* Remove element(s) from a set */
	NFSET_MSG_TEST,		/* Test an element in a set */

	NFSET_MSG_MAX
};

/* Create set */
enum nfset_attr_create_type {
	NFSET_CREATE_UNSPEC,
	NFSET_CREATE_NAME,	/* Name of the set */
	NFSET_CREATE_TYPENAME,	/* Typename of the set */
	NFSET_CREATE_FAMILY,	/* Family */
	NFSET_CREATE_INDEX,	/* Set index - at header listing */
	NFSET_CREATE_DATA,	/* Data: type specific create data */
	__NFSET_CREATE_MAX
};
#define NFSET_CREATE_MAX (__NFSET_CREATE_MAX - 1)

/* Destroy/flush/list header/list set */
enum nfset_attr_list_type {
	NFSET_LIST_UNSPEC,
	NFSET_LIST_NAME,	/* Name of the set */
	__NFSET_LIST_MAX
};
#define NFSET_LIST_MAX (__NFSET_LIST_MAX - 1)

/* Rename/swap sets */
enum nfset_attr_rename_type {
	NFSET_RENAME_UNSPEC,
	NFSET_RENAME_FROM,	/* Original set name */
	NFSET_RENAME_TO,	/* Rename/swap to name */
	__NFSET_RENAME_MAX
};
#define NFSET_RENAME_MAX (__NFSET_RENAME_MAX - 1)

/* Core adt data */
enum nfset_att_adt_core_type {
	NFSET_ADT_UNSPEC,
	NFSET_ADT_INDEX,	/* Set index */
	NFSET_ADT_DATA,	/* Type specific data */
	__NFSET_ADT_MAX
};
#define NFSET_ADT_MAX (__NFSET_ADT_MAX - 1)

#if 0
/* Type speficic adt data */
enum nfset_attr_adt_type {
	NFSET_ADT_UNSPEC,
	NFSET_ADT_IP,		/* IP address */
	NFSET_ADT_NETMASK,	/* Netmask */
	NFSET_ADT_PORT,		/* Port number */
	NFSET_ADT_IP_SECOND,	/* Second IP address */
	NFSET_ADT_MAC,		/* MAC address */
	NFSET_ADT_TIMEOUT,	/* Timeout */
	__NFSET_ADT_MAX
};
#define NFSET_ADT_MAX (__NFSET_ADT_MAX - 1)
#endif

#define NFA_PUT_SIMPLE(skb, attrtype, type, data)		\
do {								\
	struct nfattr *__nfa; 					\
	int __size = NFA_LENGTH(sizeof(type));			\
	type *__data;						\
								\
	if (skb_tailroom(skb) < (int)NFA_ALIGN(__size))		\
		goto nfattr_failure;				\
	__nfa = (struct nfattr *)skb_put(skb, NFA_ALIGN(__size));\
	__nfa->nfa_type = attrtype;				\
	__nfa->nfa_len  = __size;				\
	__data = (type *)NFA_DATA(__nfa);			\
	*__data = data;						\
	memset(NFA_DATA(__nfa) + sizeof(type), 0, 		\
	       NFA_ALIGN(__size) - __size);			\
} while (0)

/* Check attribute existence and size:
 *
 * - cta_min[i] == 0	: optional attribute
 * - cta_min[i] < 0	: mandatory attribute, don't check size
 * - cta_min[i] > 0	: mandatory attribute, check size
 */

#define nfattr_invalid_size(tb, max, cta_min)				\
({	int __i, __ret = 0;						\
 	for (__i = 0; __i < max; __i++) {				\
 		if (!cta_min[__i])					\
 			continue;					\
 		if (!tb[__i] 						\
 		    || (cta_min[__i] > 0 				\
 		        && NFA_PAYLOAD(tb[__i]) < cta_min[__i])) {	\
 			__ret = 1;					\
 			break;						\
 		}							\
 	}								\
 	__ret;								\
})



#endif /* _NF_SET_NETLINK_H */
